package fixedCode;

import java.awt.Button;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Event;
import java.awt.Font;
import java.awt.Rectangle;

import javax.swing.JFrame;

public class ControlPanel extends JFrame {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private Kernel kernel;
	private Button runButton = new Button("run");
	private Button stepButton = new Button("step");
	private Button resetButton = new Button("reset");
	private Button[] b = new Button[Constants.MAXIMUM_NUMBER_OF_VIRTUAL_PAGES];
	private String status = "STOP";

	GatedLabel statusValueLabel = new GatedLabel("STOP", GatedLabel.LEFT);
	GatedLabel timeValueLabel = new GatedLabel("0", GatedLabel.LEFT);
	GatedLabel instructionValueLabel = new GatedLabel("NONE", GatedLabel.LEFT);
	GatedLabel addressValueLabel = new GatedLabel("NULL", GatedLabel.LEFT);
	GatedLabel pageFaultValueLabel = new GatedLabel("NO", GatedLabel.LEFT);
	GatedLabel virtualPageValueLabel = new GatedLabel("x", GatedLabel.LEFT);
	GatedLabel physicalPageValueLabel = new GatedLabel("0", GatedLabel.LEFT);
	GatedLabel RValueLabel = new GatedLabel("0", GatedLabel.LEFT);
	GatedLabel MValueLabel = new GatedLabel("0", GatedLabel.LEFT);
	GatedLabel inMemTimeValueLabel = new GatedLabel("0", GatedLabel.LEFT);
	GatedLabel lastTouchTimeValueLabel = new GatedLabel("0", GatedLabel.LEFT);
	GatedLabel lowValueLabel = new GatedLabel("0", GatedLabel.LEFT);
	GatedLabel highValueLabel = new GatedLabel("0", GatedLabel.LEFT);
	
	private ControlPanelMenu menu;

	private static final int NUMBER_OF_COLUMNS = 8;

	private GatedLabel[] l = new GatedLabel[Constants.MAXIMUM_NUMBER_OF_VIRTUAL_PAGES];

	public ControlPanel() {
		this("");
		
	}

	public ControlPanel(String title) {
		super(title);
		this.setDefaultCloseOperation(EXIT_ON_CLOSE);
		menu = new ControlPanelMenu();
		this.setJMenuBar(menu);
		for (int index = 0; index < Constants.MAXIMUM_NUMBER_OF_VIRTUAL_PAGES; index++) {
			b[index] = new Button("page " + index);
		}

		for (int index = 0; index < Constants.MAXIMUM_NUMBER_OF_VIRTUAL_PAGES; index++) {
			l[index] = new GatedLabel(null, GatedLabel.CENTER);
		}
	}

	public void init(Kernel useKernel) {
		kernel = useKernel;
		kernel.setControlPanel(this);
		this.menu.setMyKernel(useKernel);
		
		setLayout(null);
		setBackground(Color.white);
		setForeground(Color.black);
		setSize(new Dimension(1350, 645));
		setFont(new Font("Courier", 0, 12));

		runButton.setForeground(Color.blue);
		runButton.setBackground(Color.lightGray);
		runButton.setBounds(new Rectangle(0, 25, 70, 15));
		add(runButton);

		stepButton.setForeground(Color.blue);
		stepButton.setBackground(Color.lightGray);
		stepButton.setBounds(new Rectangle(70, 25, 70, 15));
		add(stepButton);

		resetButton.setForeground(Color.blue);
		resetButton.setBackground(Color.lightGray);
		resetButton.setBounds(new Rectangle(140, 25, 70, 15));
		add(resetButton);

		int rowsPerColumn = Constants.MAXIMUM_NUMBER_OF_VIRTUAL_PAGES
				/ NUMBER_OF_COLUMNS;

		// Set up the virtual page labels appropriately.
		for (int index = 0; index < Constants.MAXIMUM_NUMBER_OF_VIRTUAL_PAGES; index++) {
			int x = 140 * (index / rowsPerColumn);
			int y = ((index % rowsPerColumn) + 2) * 15 + 25;
			b[index].setBounds(new Rectangle(x, y, 70, 15));
			b[index].setForeground(Color.magenta);
			b[index].setBackground(Color.lightGray);
			add(b[index]);

			// Now create the rectangles for the physical page descriptions.
			l[index].setBounds(new Rectangle(x + 70, y, 60, 15));
			l[index].setForeground(Color.red);
			l[index].setFont(new Font("Courier", 0, 10));
			add(l[index]);
		}

		// Place the labels on the screen.
		int statusOffset = NUMBER_OF_COLUMNS * 140 + 5;

		GatedLabel statusGatedLabel = new GatedLabel("status: ",
				GatedLabel.LEFT);
		statusGatedLabel.setBounds(new Rectangle(statusOffset, 0 + 25, 65, 15));
		add(statusGatedLabel);

		GatedLabel timeGatedLabel = new GatedLabel("time: ", GatedLabel.LEFT);
		timeGatedLabel.setBounds(new Rectangle(statusOffset, 15 + 25, 50, 15));
		add(timeGatedLabel);

		GatedLabel instructionGatedLabel = new GatedLabel("instruction: ",
				GatedLabel.LEFT);
		instructionGatedLabel.setBounds(new Rectangle(statusOffset, 45 + 25,
				100, 15));
		add(instructionGatedLabel);

		GatedLabel addressGatedLabel = new GatedLabel("address: ",
				GatedLabel.LEFT);
		addressGatedLabel
				.setBounds(new Rectangle(statusOffset, 60 + 25, 85, 15));
		add(addressGatedLabel);

		GatedLabel pageFaultGatedLabel = new GatedLabel("page fault: ",
				GatedLabel.LEFT);
		pageFaultGatedLabel.setBounds(new Rectangle(statusOffset, 90 + 25, 100,
				15));
		add(pageFaultGatedLabel);

		GatedLabel virtualPageGatedLabel = new GatedLabel("virtual page: ",
				GatedLabel.LEFT);
		virtualPageGatedLabel.setBounds(new Rectangle(statusOffset, 120 + 25,
				110, 15));
		add(virtualPageGatedLabel);

		GatedLabel physicalPageGatedLabel = new GatedLabel("physical page: ",
				GatedLabel.LEFT);
		physicalPageGatedLabel.setBounds(new Rectangle(statusOffset, 135 + 25,
				110, 15));
		add(physicalPageGatedLabel);

		GatedLabel RGatedLabel = new GatedLabel("Read: ", GatedLabel.LEFT);
		RGatedLabel.setBounds(new Rectangle(statusOffset, 150 + 25, 110, 15));
		add(RGatedLabel);

		GatedLabel MGatedLabel = new GatedLabel("Modified: ", GatedLabel.LEFT);
		MGatedLabel.setBounds(new Rectangle(statusOffset, 165 + 25, 110, 15));
		add(MGatedLabel);

		GatedLabel inMemTimeGatedLabel = new GatedLabel("inMemTime: ",
				GatedLabel.LEFT);
		inMemTimeGatedLabel.setBounds(new Rectangle(statusOffset, 180 + 25,
				110, 15));
		add(inMemTimeGatedLabel);

		GatedLabel lastTouchTimeGatedLabel = new GatedLabel("lastTouchTime: ",
				GatedLabel.LEFT);
		lastTouchTimeGatedLabel.setBounds(new Rectangle(statusOffset, 195 + 25,
				110, 15));
		add(lastTouchTimeGatedLabel);

		GatedLabel lowGatedLabel = new GatedLabel("low: ", GatedLabel.LEFT);
		lowGatedLabel.setBounds(new Rectangle(285, 210 + 25, 110, 15));
		add(lowGatedLabel);

		GatedLabel highGatedLabel = new GatedLabel("high: ", GatedLabel.LEFT);
		highGatedLabel
				.setBounds(new Rectangle(statusOffset, 225 + 25, 110, 15));
		add(highGatedLabel);

		// Now place the data holders
		statusOffset = NUMBER_OF_COLUMNS * 140 + 115;

		statusValueLabel
				.setBounds(new Rectangle(statusOffset, 0 + 25, 100, 15));
		add(statusValueLabel);

		timeValueLabel.setBounds(new Rectangle(statusOffset, 15 + 25, 100, 15));
		add(timeValueLabel);

		instructionValueLabel.setBounds(new Rectangle(statusOffset, 45 + 25,
				100, 15));
		add(instructionValueLabel);

		addressValueLabel.setBounds(new Rectangle(statusOffset, 60 + 25, 230,
				15));
		add(addressValueLabel);

		pageFaultValueLabel.setBounds(new Rectangle(statusOffset - 10, 90 + 25,
				100, 15));
		add(pageFaultValueLabel);

		virtualPageValueLabel.setBounds(new Rectangle(statusOffset, 120 + 25,
				200, 15));
		add(virtualPageValueLabel);

		physicalPageValueLabel.setBounds(new Rectangle(statusOffset, 135 + 25,
				200, 15));
		add(physicalPageValueLabel);

		RValueLabel.setBounds(new Rectangle(statusOffset, 150 + 25, 200, 15));
		add(RValueLabel);

		MValueLabel.setBounds(new Rectangle(statusOffset, 165 + 25, 200, 15));
		add(MValueLabel);

		inMemTimeValueLabel.setBounds(new Rectangle(statusOffset, 180 + 25,
				200, 15));
		add(inMemTimeValueLabel);

		lastTouchTimeValueLabel.setBounds(new Rectangle(statusOffset, 195 + 25,
				200, 15));
		add(lastTouchTimeValueLabel);

		lowValueLabel.setBounds(new Rectangle(statusOffset, 210 + 25, 230, 15));
		add(lowValueLabel);

		highValueLabel
				.setBounds(new Rectangle(statusOffset, 225 + 25, 230, 15));
		add(highValueLabel);

		// Column Headings...
		for (int index = 0; index < ControlPanel.NUMBER_OF_COLUMNS; index++) {
			int x = 140 * index;
			GatedLabel virtualOneGatedLabel = new GatedLabel("virtual",
					GatedLabel.CENTER);
			virtualOneGatedLabel.setBounds(new Rectangle(x, 15 + 25, 70, 15));
			add(virtualOneGatedLabel);

			x = 140 * index + 70;

			GatedLabel physicalOneGatedLabel = new GatedLabel("physical",
					GatedLabel.CENTER);
			physicalOneGatedLabel.setBounds(new Rectangle(x, 15 + 25, 70, 15));
			add(physicalOneGatedLabel);

		}
		kernel.newinit();
		//kernel.readDINFile(commands);
		this.setVisible(true);
	}

	public void paintPage(Page page) {
		virtualPageValueLabel.setText(Integer.toString(page.getId()));
		physicalPageValueLabel.setText(Integer.toString(page.getPhysical()));

		if (page.isRead() == true) {
			RValueLabel.setText("YES");
		} else {
			RValueLabel.setText("NO");
		}

		if (page.isModified() == true) {
			MValueLabel.setText("YES");
		} else {
			MValueLabel.setText("NO");
		}
		inMemTimeValueLabel.setText(Long.toString(page
				.getLengthOfTimePageIsLoaded()));
		lastTouchTimeValueLabel.setText(Long.toString(page
				.getTimePeriodSinceLastTouch()));
		lowValueLabel
				.setText(Long.toString(page.getLow(), Kernel.addressradix));
		highValueLabel.setText(Long.toString(page.getHigh(),
				Kernel.addressradix));
	}

	public void setStatus(String status) {
		statusValueLabel.setText(status);
		this.status = status;
	}
	
	public void resetPhysicalPages()
	{
		for (int index = 0; index < l.length; index++)
		{
			removePhysicalPage(index);
		}
	}

	public void addPhysicalPage(int pageNum, int physicalPage) {
		if (physicalPage < l.length) {
			l[physicalPage].setText("page" + pageNum);
		} else {
			return;
		}
	}

	public void removePhysicalPage(int physicalPage) {
		if (physicalPage < l.length) {
			l[physicalPage].setText(null);
		} else {
			return;
		}
	}

	public void simulationStopped() {
		// Cause the system to pause. TBD.
		kernel.pauseSimulation();
		runButton.setLabel("Run");
		runButton.setEnabled(true);
		stepButton.setEnabled(true);
		resetButton.setEnabled(true);
		setStatus("STOP");
	}

	public boolean action(Event e, Object arg) {
		boolean retVal = false;
		if (e.target == runButton) {
			if (this.status.equalsIgnoreCase("STOP")) {
				setStatus("RUN");
				runButton.setLabel("Stop");
				runButton.setEnabled(true);
				stepButton.setEnabled(false);
				resetButton.setEnabled(false);

				Thread t = new Thread(kernel);
				t.start();
				// kernel.run();
				// setStatus("STOP");
				// resetButton.setEnabled(true);
				retVal = true;
			} else if (this.status.equalsIgnoreCase("RUN")) {
				// Cause the system to pause. TBD.
				kernel.pauseSimulation();
				runButton.setLabel("Run");
				runButton.setEnabled(true);
				stepButton.setEnabled(true);
				resetButton.setEnabled(true);
				setStatus("STOP");
			}

		} else if (e.target == stepButton) {
			setStatus("STEP");
			kernel.step();
			if (kernel.runcycles == kernel.runs) {
				stepButton.setEnabled(false);
				runButton.setEnabled(false);
			}
			setStatus("STOP");
			retVal = true;
		} else if (e.target == resetButton) {
			kernel.reset();
			TimerTick.resetTick();
			runButton.setEnabled(true);
			stepButton.setEnabled(true);
			retVal = true;
		} else {
			for (int index = 0; index < Constants.MAXIMUM_NUMBER_OF_VIRTUAL_PAGES; index++) {
				if (e.target == b[index]) {
					kernel.getPage(index);
					retVal = true;
				}
			}
		}
		return retVal;
	}
}
