package fixedCode;
import javax.swing.*;
import java.awt.event.*;

public class ControlPanelMenu extends JMenuBar {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	private JMenu file;
	private JMenu config;
	private Kernel myKernel;
	// Create a file chooser
	private final JFileChooser fc = new JFileChooser();

	public void setMyKernel(Kernel myKernel) {
		this.myKernel = myKernel;
	}

	public ControlPanelMenu() {
		file = new JMenu("File");
		file.setMnemonic('F');
		JMenuItem openItem = new JMenuItem("Select Trace");
		openItem.setMnemonic('S');
		file.add(openItem);
		JMenuItem exitItem = new JMenuItem("Exit");
		exitItem.setMnemonic('x');
		file.add(exitItem);

		this.add(file);

		// adding action listener to menu items
		openItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {

				// In response to a button click:
				int returnVal = fc.showOpenDialog(null);
				if (returnVal == JFileChooser.APPROVE_OPTION) {
					myKernel.setAddressTraceFile(fc.getSelectedFile().getName());
					myKernel.reset();
				}
			}
		});
		exitItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				System.exit(0);
			}
		});

		config = new JMenu("Config");
		config.setMnemonic('F');

		JCheckBoxMenuItem cbMenuItem = new JCheckBoxMenuItem("Logging Enabled");
		cbMenuItem.setMnemonic(KeyEvent.VK_C);

		ActionListener aListener = new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				AbstractButton aButton = (AbstractButton) event.getSource();
				boolean selected = aButton.getModel().isSelected();
				myKernel.setDoStdoutLog(selected);
			}
		};

		cbMenuItem.addActionListener(aListener);

		config.add(cbMenuItem);

		config.addSeparator();

		JMenuItem physicalItem = new JMenuItem("Physical Memory Size");
		physicalItem.setMnemonic('P');
		config.add(physicalItem);

		// adding action listener to menu items
		physicalItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				boolean valid = false;
				while (valid == false) {
					long physMemorySize = 1024 * ConfigurationParameterDialogBoxes.getPhysicalMemorySize();
					if (physMemorySize < 0) {
						// Abort.
						valid = true;
					} else if ((physMemorySize / 2 >= myKernel.getFrameSize()) &&
							(physMemorySize % myKernel.getFrameSize()==0)){
						// Valid page size has been passed in.
						myKernel.setPhysicalMemorySize(physMemorySize);
						myKernel.reset();
						valid = true;

					} else {
						// custom title, warning icon
						JOptionPane
								.showMessageDialog(
										null,
										"Error: The physical memory size must be large enough to allow for at least 2 full frames to be loaded.  Physical memory also must be equally divisible by the frame size.",
										"Memory Warning",
										JOptionPane.WARNING_MESSAGE);
					}
				}
			}
		});

		JMenuItem frameItem = new JMenuItem("Frame Size");
		frameItem.setMnemonic('F');
		config.add(frameItem);

		// adding action listener to menu items
		frameItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {

				boolean valid = false;
				while (valid == false) {
					long frameSize = 1024 * ConfigurationParameterDialogBoxes
							.getFrameSize();
					if (frameSize < 0) {
						// Abort.
						valid = true;
					} else if ((myKernel.getPhysicalMemorySize() / 2 >= frameSize)&&
							((myKernel.getPhysicalMemorySize() % frameSize)==0)){
						// Valid page size has been passed in.
						myKernel.setFrameSize(frameSize);
						myKernel.reset();
						valid = true;

					} else {
						// custom title, warning icon
						JOptionPane
								.showMessageDialog(
										null,
										"Error: The physical memory size must be large enough to allow for at least 2 full frames to be loaded.\nThe size must also be evenly divisible by the frame size.",
										"Memory Warning",
										JOptionPane.WARNING_MESSAGE);
					}
				}
			};
		});
		this.add(config);

	}
}