package fixedCode;
import java.lang.Thread;
import java.io.*;
import java.util.*;

import javax.swing.JOptionPane;


public class Kernel extends Thread {
	// The number of virtual pages must be fixed at
	// Constants.MAXIMUM_NUMBER_OF_VIRTUAL_PAGES-1 due to
	// dependencies in the GUI
	private static int virtPageNum = Constants.MAXIMUM_NUMBER_OF_VIRTUAL_PAGES - 1;
	private static int VIRTUAL_ADDRESS_SPACE_SIZE = 1048576; // 2 ^ 20

	private String output = "ResultsLog.txt";
	// private static final String lineSeparator =
	// System.getProperty("line.separator");
	private String addressTraceFile;
	private ControlPanel controlPanel;
	private Hashtable<Integer, Page> memVector = new Hashtable<Integer, Page>();
	private Vector<Instruction> instructVector = new Vector<Instruction>();
	private boolean doStdoutLog = false;
	private boolean doFileLog = false;
	public int runs;
	public int runcycles;
	public long frameSize = (int) Math.pow(2, 12);
	public static byte addressradix = 16;
	private Page lastReferencedPage = null;

	private long physicalMemorySize = 1024*8;

	public long getPhysicalMemorySize() {
		return physicalMemorySize;
	}

	public void setPhysicalMemorySize(long physicalMemorySize) {
		this.physicalMemorySize = physicalMemorySize;
	}

	public String getAddressTraceFile() {
		return addressTraceFile;
	}

	public void setAddressTraceFile(String addressTraceFile) {
		this.addressTraceFile = addressTraceFile;
	}

	public boolean isDoStdoutLog() {
		return doStdoutLog;
	}

	public void setDoStdoutLog(boolean doStdoutLog) {
		this.doStdoutLog = doStdoutLog;
	}

	public long getFrameSize() {
		return frameSize;
	}

	public void setFrameSize(long frameSize) {
		this.frameSize = frameSize;
	}

	private boolean continueSimulation = true;

	public void pauseSimulation() {
		continueSimulation = false;
	}

	// This variable really shouldn't be like this.
	// TODO Fix this.
	long address_limit;

	/**
	 * This method will read and import a din format file for processing.
	 * 
	 * @param filename
	 *            This is the filename that is to be processed.
	 * @throws FileNotFoundException
	 */
	public void readDINFile(String filename) {
		try {

			File f = new File(filename);
			addressTraceFile = filename;

			Scanner in = new Scanner(f);
			in.useRadix(16);

			int lcount = 0;
			while (in.hasNext() == true) {

				lcount++;
				if ((lcount % 100) == 0) {
					System.out.printf("Line %d\n", lcount);
				}
				long type;
				long address;
				String command;

				type = in.nextLong();
				address = in.nextLong();

				long addr = compressAddress(address);

				if ((type == 0) || (type == 2)) {
					command = "READ";
				} else if (type == 1) {
					command = "WRITE";
				} else {
					command = "ERROR";
				}
				if (command.equalsIgnoreCase("ERROR")) {
					// DO nothing.
				} else {
					instructVector.addElement(new Instruction(command, addr));
				}
			}
			in.close();
		} catch (IOException e) { /* Handle exceptions */
		}
	}

	public long compressAddress(long inputAddress) {
		long retVal;

		long msNibble = inputAddress & 0xF0000000;
		long lsNibble = inputAddress & 0x0000000F;
		long a = inputAddress & 0x0FFF0000;
		long b = inputAddress & 0x0000FFF0;

		retVal = (a >> 12) ^ b;
		retVal = retVal | lsNibble;
		retVal = retVal | (msNibble >> 16);

		return retVal;
	}

	public void newinit() {
		int i = 0;
		int j = 0;
		int physical_count = 0;
		int map_count = 0;
		long high = 0;
		long low = 0;
		
		TimerTick.resetTick();
		controlPanel.resetPhysicalPages();
		
		virtPageNum = (int) ((VIRTUAL_ADDRESS_SPACE_SIZE - 1) / frameSize);
		address_limit = (frameSize * (virtPageNum + 1)) - 1;
		int physicalPageCount = (int) (this.physicalMemorySize / frameSize);

		System.out.println("Physical Memory: " + this.physicalMemorySize
				+ " Block size: " + frameSize + "Limit: " + address_limit);

		for (i = 0; i <= virtPageNum; i++) {
			high = (frameSize * (i + 1)) - 1;
			low = frameSize * i;
			Page p = new Page(i, -1, false, false, 0, 0, high, low);
			p.setHigh((frameSize * (i + 1)) - 1);
			p.setLow(frameSize * i);
			memVector.put(i, p);
		}

		if (doFileLog) {
			File trace = new File(output);
			trace.delete();
		}
		runs = 0;
		for (i = 0; i < virtPageNum; i++) {
			Page page = memVector.get(i);
			if (page.physical != -1) {
				map_count++;
			}
			for (j = 0; j < virtPageNum; j++) {
				Page tmp_page = memVector.get(j);
				if (tmp_page.physical == page.physical && page.physical >= 0) {
					physical_count++;
				}
			}
			physical_count = 0;
		}
		if (map_count < physicalPageCount) {
			for (i = 0; i < virtPageNum; i++) {
				Page page = memVector.get(i);
				if (page.physical == -1 && map_count < physicalPageCount) {
					page.physical = i;
					map_count++;
				}
			}
		}
		for (i = 0; i < virtPageNum; i++) {
			Page page = memVector.get(i);
			if (page.physical == -1) {
				controlPanel.removePhysicalPage(i);
			} else {
				controlPanel.addPhysicalPage(i, page.physical);
			}
		}
		for (i = 0; i < instructVector.size(); i++) {
			high = frameSize * (virtPageNum + 1);
			Instruction instruct = (Instruction) instructVector.elementAt(i);
			if (instruct.getAddr() < 0 || instruct.getAddr() > high) {
				System.out
						.printf("MemoryManagement: Instruction (%s %x) out of bounds. Range: %x %x\n",
								instruct.getInst(), instruct.getAddr(), low,
								high);
				System.exit(-1);
			}
		}
	}

	public void setControlPanel(ControlPanel newControlPanel) {
		controlPanel = newControlPanel;
	}

	public void getPage(int pageNum) {
		Page page = memVector.get(pageNum);
		controlPanel.paintPage(page);
	}

	private void printPageFaultCount() {
		// File trace = new File(output);

		try {
			PrintStream out = new PrintStream(
					new FileOutputStream(output, true));

			out.println("Trace File Name: " + this.addressTraceFile);
			out.println("Frame Size:" + this.frameSize);
			out.println("Physical Memory Size: " + this.physicalMemorySize);
			out.println("Number of page missed: " + PageFault.faultCount);
			out.println("Total Memory Accesses: " + instructVector.size());
			out.close();
		} catch (IOException e) {
			/* Do nothing */
		}
	}

	private PrintStream out;

	public void openFileLog() {
		try {
			out = new PrintStream(new FileOutputStream(output, true));
		} catch (Exception ex) {
			System.err.println("Error opening log file.");
			ex.printStackTrace();
			System.exit(-1);
		}
	}

	public void closeFileLog() {
		out.close();
	}

	private void printLogFile(String message) {
		out.println(message);
	}

	public void run() {
		openFileLog();
		continueSimulation = true;

		// TODO WWS Implement this in a more graceful fashion, maybe even to the
		// point of not needing it here...
		runcycles = instructVector.size();
		if (runcycles < 1) {
			System.out
					.println("MemoryManagement: no instructions present for execution.");
			System.exit(-1);
		}
		performSingleMemoryStep();
		GatedLabel.inhibitAllUpdates();
		int count = 0;
		while ((runs != runcycles) && (continueSimulation == true)) {
			performSingleMemoryStep();
			count++;
			
			if (count % 255 == 0)
			{
				GatedLabel.refreshAll();
			}
		}
		printPageFaultCount();
		GatedLabel.allAllUpdates();
		GatedLabel.refreshAll();
		closeFileLog();
		controlPanel.simulationStopped();
		
		String message = "Simulation stopped.\n";
		message+= "Trace file name: " + this.addressTraceFile + "\n";
		message+= "Frame Size:" + this.frameSize+ "\n";
		message+= "Physical Memory Size: " + this.physicalMemorySize+ "\n";
		message+= "Number of page missed: " + PageFault.faultCount+ "\n";
		message+= "Total Memory Accesses: " + instructVector.size()+ "\n";
		
		JOptionPane.showMessageDialog(null, message, "Simulation Status", JOptionPane.INFORMATION_MESSAGE);
	}

	public void step() {
		openFileLog();
		performSingleMemoryStep();
		closeFileLog();
	}

	private void performSingleMemoryStep() {
		Page currentPage = null;

		Instruction instruct = instructVector.elementAt(runs);
		controlPanel.instructionValueLabel.setText(instruct.getInst());
		controlPanel.addressValueLabel.setText(Long.toString(
				instruct.getAddr(), addressradix));
		getPage(Virtual2Physical.pageNum(instruct.getAddr(), virtPageNum,
				frameSize));
		if (controlPanel.pageFaultValueLabel.getText() == "YES") {
			controlPanel.pageFaultValueLabel.setText("NO");
		}
		if (instruct.isRead()) {
			currentPage = handleReadInstruction(instruct);
		} else if (instruct.isWrite()) {
			currentPage = handleWriteInstruction(instruct);
		}

		// Reset the last read page.
		if (this.lastReferencedPage != null) {
			if ((this.lastReferencedPage.isRead() == true)
					&& (this.lastReferencedPage.getTimePeriodSinceLastTouch() == 10)) {
				this.lastReferencedPage.setRead(false);
			}
		}

		this.lastReferencedPage = currentPage;

		runs++;
		TimerTick.timerTick();

		controlPanel.timeValueLabel.setText(TimerTick.getTimerTick() + " (ns)");
		// perf.printTimes();
	}

	private Page handleWriteInstruction(Instruction instruct) {

		Page page = memVector.get(Virtual2Physical.pageNum(instruct.getAddr(),
				virtPageNum, frameSize));
		if (page.getPhysical() == -1) {
			if (doFileLog) {
				printLogFile("WRITE "
						+ Long.toString(instruct.getAddr(), addressradix)
						+ " ... page fault");
			}
			if (doStdoutLog) {
				System.out.println("WRITE "
						+ Long.toString(instruct.getAddr(), addressradix)
						+ " ... page fault");
			}
			PageFault.replacePage(memVector, virtPageNum, Virtual2Physical
					.pageNum(instruct.getAddr(), virtPageNum, frameSize),
					controlPanel);
			controlPanel.pageFaultValueLabel.setText("YES");
			TimerTick.swapTimer();

		} else {
			page.setModified(true);
			page.setLastTouchTime(TimerTick.getTimerTick());
			if (doFileLog) {
				printLogFile("WRITE "
						+ Long.toString(instruct.getAddr(), addressradix)
						+ " ... okay");
			}
			if (doStdoutLog) {
				System.out.println("WRITE "
						+ Long.toString(instruct.getAddr(), addressradix)
						+ " ... okay");
			}
		}

		return page;
	}

	private Page handleReadInstruction(Instruction instruct) {
		Page page = (Page) memVector.get(Virtual2Physical.pageNum(
				instruct.getAddr(), virtPageNum, frameSize));
		if (page.physical == -1) {
			if (doFileLog) {
				printLogFile("READ "
						+ Long.toString(instruct.getAddr(), addressradix)
						+ " ... page fault");
			}

			if (doStdoutLog) {
				System.out.println("READ "
						+ Long.toString(instruct.getAddr(), addressradix)
						+ " ... page fault");
			}

			PageFault.replacePage(memVector, virtPageNum, Virtual2Physical
					.pageNum(instruct.getAddr(), virtPageNum, frameSize),
					controlPanel);
			TimerTick.swapTimer();

			controlPanel.pageFaultValueLabel.setText("YES");

		} else {
			page.setRead(true);
			page.setLastTouchTime(TimerTick.getTimerTick());
			if (doFileLog) {
				printLogFile("READ "
						+ Long.toString(instruct.getAddr(), addressradix)
						+ " ... okay");
			}

			if (doStdoutLog) {
				System.out.println("READ "
						+ Long.toString(instruct.getAddr(), addressradix)
						+ " ... okay");
			}

		}
		return page;
	}

	public void reset() {
		memVector.clear();
		instructVector.removeAllElements();
		controlPanel.statusValueLabel.setText("STOP");
		controlPanel.timeValueLabel.setText("0");
		controlPanel.instructionValueLabel.setText("NONE");
		controlPanel.addressValueLabel.setText("NULL");
		controlPanel.pageFaultValueLabel.setText("NO");
		controlPanel.virtualPageValueLabel.setText("x");
		controlPanel.physicalPageValueLabel.setText("0");
		controlPanel.RValueLabel.setText("0");
		controlPanel.MValueLabel.setText("0");
		controlPanel.inMemTimeValueLabel.setText("0");
		controlPanel.lastTouchTimeValueLabel.setText("0");
		controlPanel.lowValueLabel.setText("0");
		controlPanel.highValueLabel.setText("0");
		PageFault.faultCount = 0;
		newinit();
		if (this.addressTraceFile != null) {
			this.readDINFile(this.addressTraceFile);
		}
	}
}
